@@ -14,7 +14,9 @@
     timeout=30L * 24L * 60L * 60L, # 30 days
     exportglobals=TRUE,
     progressbar=FALSE,
-    RNGseed=NULL
+    RNGseed=NULL,
+    RNGstream = NULL,
+    force.GC = FALSE
 )
 
 .BiocParallelParam <- setRefClass("BiocParallelParam",
@@ -32,7 +34,9 @@
         stop.on.error="logical",
         timeout="integer",
         exportglobals="logical",
-        RNGseed = "ANY",
+        RNGseed = "ANY",        # NULL or integer(1)
+        RNGstream = "ANY",      # NULL or integer(); internal use only
+        force.GC = "logical",
         ## cluster management
         .finalizer_env = "environment",
         .uid = "character"
@@ -55,6 +59,7 @@
                 "; bpprogressbar: ", bpprogressbar(.self),
                 "\n",
                 "  bpexportglobals: ", bpexportglobals(.self),
+                "; bpforceGC: ", bpforceGC(.self),
                 "\n", .prettyPath("  bplogdir", bplogdir(.self)),
                 "\n", .prettyPath("  bpresultdir", bpresultdir(.self)),
                 "\n", sep="")
@@ -79,7 +84,7 @@ setValidity("BiocParallelParam", function(object)
     if (!is.numeric(tasks))
         msg <- c(msg, "bptasks(BPPARAM) must be an integer")
     if (length(tasks) > 1L)
-        msg <- c(msg, "length(bpwtasks(BPPARAM)) must be == 1") 
+        msg <- c(msg, "length(bptasks(BPPARAM)) must be == 1")
 
     if (is.character(workers)) {
         if (length(workers) < 1L)
@@ -124,7 +129,10 @@ setValidity("BiocParallelParam", function(object)
     }
 
     if (!.isTRUEorFALSE(bpstopOnError(object)))
-        msg <- c(msg, "'bpstopOnError' must be logical(1)")
+        msg <- c(msg, "'bpstopOnError' must be TRUE or FALSE")
+
+    if (!.isTRUEorFALSE(bpforceGC(object)))
+        msg <- c(msg, "'force.GC' must be TRUE or FALSE")
 
     if (is.null(msg)) TRUE else msg
 })
@@ -261,7 +269,8 @@ setMethod("bpprogressbar", "BiocParallelParam",
 setReplaceMethod("bpprogressbar", c("BiocParallelParam", "logical"),
     function(x, value)
 {
-    x$progressbar <- value 
+    x$progressbar <- value
+    validObject(x)
     x
 })
 
@@ -275,6 +284,7 @@ setReplaceMethod("bpRNGseed", c("BiocParallelParam", "NULL"),
     function(x, value)
 {
     x$RNGseed <- NULL
+    validObject(x)
     x
 })
 
@@ -282,6 +292,45 @@ setReplaceMethod("bpRNGseed", c("BiocParallelParam", "numeric"),
     function(x, value)
 {
     x$RNGseed <- as.integer(value)
+    validObject(x)
+    x
+})
+
+.RNGstream <-
+    function(x)
+{
+    x$RNGstream
+}
+
+`.RNGstream<-` <-
+    function(x, value)
+{
+    value <- as.integer(value)
+    if (anyNA(value))
+        stop("[internal] RNGstream value could not be coerced to integer")
+    x$RNGstream <- value
+    x
+}
+
+.bpnextRNGstream <-
+    function(x)
+{
+    ## initialize or get the next random number stream; increment the
+    ## stream only in bpstart_impl
+    .RNGstream(x) <- .rng_next_stream(.RNGstream(x))
+}
+
+setMethod("bpforceGC", "BiocParallelParam",
+    function(x)
+{
+    x$force.GC
+})
+
+setReplaceMethod("bpforceGC", c("BiocParallelParam", "numeric"),
+    function(x, value)
+{
+    x$force.GC <- as.logical(value)
+    validObject(x)
     x
 })
 
