@@ -23,6 +23,7 @@ MulticoreParam <- function(workers=multicoreWorkers(), tasks=0L,
         exportglobals=TRUE,
         log=FALSE, threshold="INFO", logdir=NA_character_,
         resultdir=NA_character_, jobname = "BPJOB",
+        force.GC = TRUE,
         manager.hostname=NA_character_, manager.port=NA_integer_, ...)
 {
     if (.Platform$OS.type == "windows") {
@@ -30,6 +31,9 @@ MulticoreParam <- function(workers=multicoreWorkers(), tasks=0L,
         workers = 1L
     }
 
+    if (progressbar && missing(tasks))
+        tasks <- .Machine$integer.max
+
     clusterargs <- c(list(spec=workers, type="FORK"), list(...))
 
     manager.hostname <-
@@ -58,11 +62,14 @@ MulticoreParam <- function(workers=multicoreWorkers(), tasks=0L,
         exportglobals=exportglobals,
         log=log, threshold=threshold,
         logdir=logdir, resultdir=resultdir, jobname=jobname,
+        force.GC = force.GC,
         hostname=manager.hostname, port=manager.port,
         ...
     )
 
-    do.call(.MulticoreParam, prototype)
+    x <- do.call(.MulticoreParam, prototype)
+    validObject(x)
+    x
 }
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
