@@ -23,8 +23,8 @@
     port <- getOption("ports", port)
 
     if (identical(tolower(port), "random") || is.na(port)) {
-        octx <- .internal_rng_stream$set()
-        on.exit(.internal_rng_stream$unset(octx))
+        .rng_internal_stream$set()
+        on.exit(.rng_internal_stream$reset())
         port <- as.integer(
             11000 +
             1000 * ((stats::runif(1L) + unclass(Sys.time()) / 300) %% 1L)
@@ -48,12 +48,7 @@
     if (type == "multicore" && .Platform$OS.type == "windows")
         return(1L)
 
-    cores <- min(.detectCores(), .snowCoresMax(type))
-    cores <- getOption("mc.cores", cores)
-    if (nzchar(Sys.getenv("BBS_HOME")))
-        cores <- min(4L, cores)
-
-    cores
+    min(.detectCores(), .snowCoresMax(type))
 }
 
 snowWorkers <- function(type = c("SOCK", "MPI", "FORK")) {
@@ -109,6 +104,7 @@ SnowParam <- function(workers=snowWorkers(type),
                       exportglobals=TRUE,
                       log=FALSE, threshold="INFO", logdir=NA_character_,
                       resultdir=NA_character_, jobname = "BPJOB",
+                      force.GC = FALSE,
                       manager.hostname=NA_character_,
                       manager.port=NA_integer_, ...)
 {
@@ -120,6 +116,9 @@ SnowParam <- function(workers=snowWorkers(type),
     if (type %in% c("MPI", "FORK") && is(workers, "character"))
         stop("'workers' must be integer(1) when 'type' is MPI or FORK")
 
+    if (progressbar && missing(tasks))
+        tasks <- .Machine$integer.max
+
     clusterargs <- c(list(spec=workers, type=type), list(...))
 
     manager.hostname <-
@@ -146,6 +145,7 @@ SnowParam <- function(workers=snowWorkers(type),
         timeout=as.integer(timeout), exportglobals=exportglobals,
         log=log, threshold=threshold, logdir=logdir,
         resultdir=resultdir, jobname=jobname,
+        force.GC = force.GC,
         hostname=manager.hostname, port=manager.port,
         ...
     )
@@ -206,6 +206,17 @@ setReplaceMethod("bpworkers", c("SnowParam", "character"),
 ### Methods - control
 ###
 
+.bpstart_makeCluster <-
+    function(cargs)
+{
+    ## set internal stream to avoid iterating global random number
+    ## stream in `parallel::makeCluster()`. Use the internal stream so
+    ## that the random number generator advances on each call.
+    state <- .rng_internal_stream$set()
+    on.exit(.rng_internal_stream$reset())
+    do.call(parallel::makeCluster, cargs)
+}
+
 setMethod("bpstart", "SnowParam",
     function(x, lenX = bpnworkers(x))
 {
@@ -242,7 +253,7 @@ setMethod("bpstart", "SnowParam",
             cargs$port <- .port(x)
         }
 
-        bpbackend(x) <- do.call(parallel::makeCluster, cargs)
+        bpbackend(x) <- .bpstart_makeCluster(cargs)
     }
 
     .bpstart_impl(x)
