@@ -45,50 +45,33 @@ setMethod("bplapply", c("ANY", "list"),
     if (!length(X))
         return(.rename(list(), X))
 
-    if (!bpschedule(BPPARAM) || length(X) == 1L || bpnworkers(BPPARAM) == 1L) {
-        param <- SerialParam(stop.on.error=bpstopOnError(BPPARAM),
-                             log=bplog(BPPARAM),
-                             threshold=bpthreshold(BPPARAM),
-                             logdir = bplogdir(BPPARAM),
-                             progressbar=bpprogressbar(BPPARAM))
-        return(bplapply(X, FUN, ..., BPREDO=BPREDO, BPPARAM=param))
+    ## which need to be redone?
+    redo_index <- .redo_index(X, BPREDO)
+    if (any(redo_index)) {
+        compute_element <- which(redo_index)
+        X <- X[compute_element]
+    } else {
+        compute_element <- seq_along(X)
     }
-
-    idx <- .redo_index(X, BPREDO)
-    if (any(idx))
-        X <- X[idx]
     nms <- names(X)
 
-    ## start / stop cluster
-    if (!bpisup(BPPARAM)) {
-        if (is(BPPARAM, "MulticoreParam"))
-            BPPARAM <- TransientMulticoreParam(BPPARAM)
-        BPPARAM <- bpstart(BPPARAM, length(X))
-        on.exit(bpstop(BPPARAM), TRUE)
-    }
+    ## split into tasks
+    X <- .splitX(X, bpnworkers(BPPARAM), bptasks(BPPARAM), redo_index)
 
-    ## FUN
-    FUN <- .composeTry(
-        FUN, bplog(BPPARAM), bpstopOnError(BPPARAM),
-        timeout=bptimeout(BPPARAM), exportglobals=bpexportglobals(BPPARAM)
+    ARGS <- list(...)
+
+    manager <- structure(list(), class="lapply") # dispatch
+    res <- .bpinit(
+        manager = manager,
+        X = X,
+        FUN = FUN,
+        ARGS = ARGS,
+        BPPARAM = BPPARAM,
+        BPREDO = BPREDO
     )
 
-    ## split into tasks
-    X <- .splitX(X, bpnworkers(BPPARAM), bptasks(BPPARAM))
-    ARGFUN <- function(i) c(list(X=X[[i]]), list(FUN=FUN), list(...))
-
-    cls <- structure(list(), class="lapply")
-    res <- bploop(cls, X, lapply, ARGFUN, BPPARAM)
-
-    if (!is.null(res)) {
-        res <- do.call(unlist, list(res, recursive=FALSE))
-        names(res) <- nms
-    }
-
-    if (any(idx)) {
-        BPREDO[idx] <- res
-        res <- BPREDO
-    }
+    if (!is.null(res))
+        names(res)[compute_element] <- nms
 
     if (!all(bpok(res)))
         stop(.error_bplist(res))
