@@ -29,27 +29,37 @@
 ### Worker loop used by SOCK, MPI and FORK.  Error handling is done in
 ### .composeTry.
 
-.bpworker_EXEC <- function(msg)
+.bpworker_EXEC <- function(msg, sink.sout = TRUE)
 {
     ## need local handler for worker read/send errors
-    sout <- character()
-    file <- textConnection("sout", "w", local=TRUE)
-    sink(file, type="message")
-    sink(file, type="output")
+    if (sink.sout) {
+        on.exit({
+            sink(NULL, type="message")
+            sink(NULL, type="output")
+            close(file)
+        })
+        file <- rawConnection(raw(), "r+")
+        sink(file, type="message")
+        sink(file, type="output")
+    }
+
     t1 <- proc.time()
     value <- tryCatch({
         do.call(msg$data$fun, msg$data$args)
     }, error=function(e) {
-        ## capture error, without throwing
-        .error_worker_comm(e, "worker evaluation failed")
+        ## return as 'list()' because msg$data$fun has lapply semantics
+        list(.error_worker_comm(e, "worker evaluation failed"))
     })
     t2 <- proc.time()
-    sink(NULL, type="message")
-    sink(NULL, type="output")
-    close(file)
+
+    if (sink.sout) {
+        sout <- rawToChar(rawConnectionValue(file))
+        if (!nchar(sout)) sout <- NULL
+    } else {
+        sout <- NULL
+    }
 
     success <- !(inherits(value, "bperror") || !all(bpok(value)))
-
     log <- .log_buffer_get()
 
     value <- .VALUE(
@@ -86,22 +96,22 @@
 ## files and 'stop on error'. The 'cl' argument is an active cluster;
 ## starting and stopping of 'cl' is done outside these functions, eg,
 ## bplapply, bpmapply etc..
-
-.clear_cluster <- function(cl, running, result=NULL)
+.clear_cluster <- function(cl, running, reducer)
 {
     tryCatch({
         setTimeLimit(30, 30, TRUE)
         on.exit(setTimeLimit(Inf, Inf, FALSE))
         while (any(running)) {
             d <- .recv_any(cl)
-            if (!is.null(result))
-                result[[d$value$tag]] <- d$value$value
+            njob <- d$value$tag
+            value <- d$value$value
+            reducer$add(njob, value)
             running[d$node] <- FALSE
         }
     }, error=function(e) {
         warning(.error_worker_comm(e, "stop worker failed"))
     })
-    result
+    reducer
 }
 
 .manager_log <- function(BPPARAM, njob, d) {
@@ -141,10 +151,16 @@
 
     reduce_one <- function(idx) {
         if (env[["reduce"]]) {
-            env[["value"]] <- if (!env[["init"]] && (env[["index"]] == 1L)) {
-                env[[idx]]
+            if (!env[["init"]] && (env[["index"]] == 1L)) {
+                env[["value"]] <- env[[idx]][[1]]
+                skip_idx <- 1L
             } else {
-                REDUCE(env[["value"]], env[[idx]])
+                skip_idx <- 0L
+            }
+            for (i in seq_along(env[[idx]])) {
+                if (i == skip_idx)
+                    next
+                env[["value"]] <- REDUCE(env[["value"]], env[[idx]][[i]])
             }
             rm(list=idx, envir=env)
         }
@@ -177,73 +193,63 @@
         else {
             lst <- as.list(env)
             idx <- setdiff(names(lst), c("reduce", "init", "value", "index"))
-            unname(lst[idx[order(as.integer(idx))]])
+            if (length(idx)) {
+                lst <- unname(lst[idx[order(as.integer(idx))]])
+                unlist(lst, recursive=FALSE)
+            } else {
+                list()
+            }
         }
     })
 }
 
+## A dummy iterator for bploop.lapply
+.bploop_lapply_iter <-
+    function(X)
+{
+    i <- 0L
+    n <- length(X)
+    function() {
+        if (i < n) {
+            i <<- i + 1L
+            X[[i]]
+        } else {
+            list(NULL)
+        }
+    }
+}
+
+## This class object can force bploop.iterator to iterate
+## the seed stream n times
+.bploop_rng_iter <- function(n) {
+    structure(as.integer(n), class = c(".bploop_rng_iter"))
+}
+
 ##
 ## bploop.lapply(): derived from snow::dynamicClusterApply.
 ##
-
 bploop <- function(manager, ...)
     UseMethod("bploop")
 
+## X: the loop value after division
+## ARGS: The function arguments for `FUN`
 bploop.lapply <-
-    function(manager, X, FUN, ARGFUN, BPPARAM, ...)
+    function(manager, X, FUN, ARGS, BPPARAM, ...)
 {
-    cl <- bpbackend(BPPARAM)
-
-    n <- length(X)
-    workers <- length(cl)
-    result <- vector("list", n)
-
-    if (n > 0 && workers > 0) {
-        progress <- .progress(active=bpprogressbar(BPPARAM))
-        on.exit(progress$term(), TRUE)
-        progress$init(n)
-
-        ## initial load
-        running <- logical(workers)
-        for (i in seq_len(min(n, workers))) {
-            value <- .EXEC(i, FUN, ARGFUN(i))
-            running[i] <- .send_to(cl, i, value)
-        }
-
-        for (i in seq_len(n)) {
-            ## collect
-            d <- .recv_any(cl)
-
-            value <- d$value$value
-            njob <- d$value$tag
-            result[[njob]] <- value
-            running[d$node] <- FALSE
-
-            progress$step()
-            .manager_log(BPPARAM, njob, d)
-            .manager_result_save(BPPARAM, njob, value)
-
-            if (bpstopOnError(BPPARAM) && !d$value$success) {
-                ## let running jobs finish, don't re-load
-                result <- .clear_cluster(cl, running, result)
-                break
-            }
-
-            ## re-load
-            j <- i + min(n, workers)
-            if (j <= n) {
-                value <- .EXEC(j, FUN, ARGFUN(j))
-                running[d$node] <- .send_to(cl, d$node, value)
-            }
-        }
-    }
-
-    ## return results
-    if (!is.na(bpresultdir(BPPARAM)))
-        NULL
-    else result
+    ITER <- .bploop_lapply_iter(X)
+    manager <- structure(list(), class="iterate") # dispatch
+    bploop(
+        manager = manager,
+        ITER = ITER,
+        FUN = FUN,
+        ARGS = ARGS,
+        BPPARAM =BPPARAM,
+        reduce.in.order = TRUE,
+        progress.length = length(X)
+    )
 }
 
+
 ##
 ## bploop.iterate():
 ##
@@ -252,32 +258,58 @@ bploop.lapply <-
 ## - length of 'X' is unknown (defined by ITER())
 ## - results not pre-allocated; list grows each iteration if no REDUCE
 ## - args wrapped in arglist with different chunks from ITER()
-##
-
+## Arguments
+## - ITER: Return a list where each list element will be passed to FUN
+##   1. if nothing to proceed, it should return list(NULL)
+##   2. if the task is iterate the seed stream only, it should return
+##      an object from .bploop_rng_iter()
+## - FUN: A function that will be evaluated in the worker
+## - ARGS: the arguments to FUN
 bploop.iterate <-
-    function(manager, ITER, FUN, ARGFUN, BPPARAM, REDUCE, init, reduce.in.order,
-             ...)
+    function(
+        manager, ITER, FUN, ARGS, BPPARAM, REDUCE, init, reduce.in.order,
+        progress.length, ...
+    )
 {
     cl <- bpbackend(BPPARAM)
 
+    seed <- .RNGstream(BPPARAM)
+    on.exit(.RNGstream(BPPARAM) <- seed)
+
     workers <- length(cl)
     running <- logical(workers)
     reducer <- .reducer(REDUCE, init, reduce.in.order)
 
-    progress <- .progress(active=bpprogressbar(BPPARAM), iterate=TRUE)
+    progress <- .progress(
+        active=bpprogressbar(BPPARAM), iterate=missing(progress.length)
+    )
     on.exit(progress$term(), TRUE)
-    progress$init()
+    progress$init(progress.length)
 
+    ARGFUN <- function(X, seed)
+        c(
+            list(X=X), list(FUN=FUN), ARGS,
+            list(BPRNGSEED = seed)
+        )
     ## initial load
     for (i in seq_len(workers)) {
         value <- ITER()
-        if (is.null(value)) {
+        ## If the value is of the class .bploop_rng_iter, we merely iterate
+        ## the seed stream `value` times and obtain the next value. There must
+        ## be no two consecutive .bploop_rng_iter objects in the iterator, so
+        ## we can proceed without checking the class again.
+        if (inherits(value, ".bploop_rng_iter")) {
+            seed <- .rng_iterate_substream(seed, value)
+            value <- ITER()
+        }
+        if (identical(value, list(NULL))) {
             if (i == 1L)
                 warning("first invocation of 'ITER()' returned NULL")
             break
         }
-        value <- .EXEC(i, FUN, ARGFUN(value))
-        running[i] <- .send_to(cl, i, value)
+        value_ <- .EXEC(i, .rng_lapply, ARGFUN(value, seed))
+        running[i] <- .send_to(cl, i, value_)
+        seed <- .rng_iterate_substream(seed, length(value))
     }
 
     repeat {
@@ -301,16 +333,21 @@ bploop.iterate <-
         if (bpstopOnError(BPPARAM) && !d$value$success) {
             ## stop on error; let running jobs finish, do not re-load
             ## FIXME: harvest assigned jobs
-            .clear_cluster(cl, running)
+            reducer <- .clear_cluster(cl, running, reducer)
             break
         }
 
         ## re-load
         value <- ITER()
-        if (!is.null(value)) {
+        if (inherits(value, "rng_iter")) {
+            seed <- .rng_iterate_substream(seed, value)
+            value <- ITER()
+        }
+        if (!identical(value, list(NULL))) {
             i <- i + 1L
-            value <- .EXEC(i, FUN, ARGFUN(value))
-            running[d$node] <- .send_to(cl, d$node, value)
+            value_ <- .EXEC(i, .rng_lapply, ARGFUN(value, seed))
+            running[d$node] <- .send_to(cl, d$node, value_)
+            seed <- .rng_iterate_substream(seed, length(value))
         }
     }
 
@@ -323,7 +360,7 @@ bploop.iterate <-
 }
 
 bploop.iterate_batchtools <-
-    function(manager, ITER, FUN, BPPARAM, REDUCE, init, reduce.in.order,...)
+    function(manager, ITER, FUN, BPPARAM, REDUCE, init, reduce.in.order, ...)
 {
     ## get number of workers
     workers <- bpnworkers(BPPARAM)
@@ -384,7 +421,7 @@ bploop.iterate_batchtools <-
     ## reduce in order
     for (job.id in ids$job.id) {
         value <- batchtools::loadResult(id = job.id, reg=BPPARAM$registry)
-        reducer$add(job.id, value)
+        reducer$add(job.id, list(value))
     }
 
     ## return reducer value
