@@ -14,12 +14,19 @@ setMethod("bpvec", c("ANY", "BiocParallelParam"),
     FUN <- match.fun(FUN)
     AGGREGATE <- match.fun(AGGREGATE)
 
-    if (!bpschedule(BPPARAM))
-        return(bpvec(X, FUN, ..., AGGREGATE=AGGREGATE, BPREDO=BPREDO,
-               BPPARAM=SerialParam()))
+    if (!bpschedule(BPPARAM)) {
+        param <- as(BPPARAM, "SerialParam")
+        return(
+            bpvec(
+                X, FUN, ..., AGGREGATE=AGGREGATE, BPREDO=BPREDO, BPPARAM = param
+            )
+        )
+    }
 
     si <- .splitX(seq_along(X), bpnworkers(BPPARAM), bptasks(BPPARAM))
-    bptasks(BPPARAM) <- 0
+    otasks <- bptasks(BPPARAM)
+    bptasks(BPPARAM) <- 0L
+    on.exit(bptasks(BPPARAM) <- otasks)
 
     idx <- .redo_index(si, BPREDO)
     if (any(idx))
