@@ -5,9 +5,17 @@ test_SerialParam_bpnworkers <- function() {
 }
 
 test_SerialParam_bpbackend <- function() {
-    checkIdentical(list(FALSE), bpbackend(SerialParam()))
-    checkIdentical(list(TRUE), bpbackend(bpstart(SerialParam())))
-    checkIdentical(list(FALSE), bpbackend(bpstop(bpstart(SerialParam()))))
+    checkIdentical(NULL, bpbackend(SerialParam()))
+    checkTrue(is(bpbackend(bpstart(SerialParam())), "SerialBackend"))
+    checkIdentical(NULL, bpbackend(bpstop(bpstart(SerialParam()))))
+}
+
+test_SerialParam_bpforceGC <- function() {
+    checkIdentical(FALSE, bpforceGC(SerialParam()))
+    checkIdentical(FALSE, bpforceGC(SerialParam(force.GC = FALSE)))
+    checkIdentical(TRUE, bpforceGC(SerialParam(force.GC = TRUE)))
+    checkException(SerialParam(force.GC = NA), silent = TRUE)
+    checkException(SerialParam(force.GC = 1:2), silent = TRUE)
 }
 
 test_SerialParam_bpisup_start_stop <- function() {
