@@ -52,7 +52,7 @@ test_SnowParam_coerce_from_SOCK <- function()
     exp <- "'bpstart' not available; instance from outside BiocParallel?"
     checkIdentical(exp, obs)
 
-    obs <- tryCatch(bpstop(p), error=conditionMessage)
+    obs <- tryCatch(bpstop(p), warning=conditionMessage)
     exp <- "'bpstop' not available; instance from outside BiocParallel?"
     checkIdentical(exp, obs)
 
@@ -105,3 +105,18 @@ test_SnowParam_workers <- function()
     checkException(SnowParam("host", "MPI"), silent=TRUE)
     checkException(SnowParam("host", "FORK"), silent=TRUE)
 }
+
+test_SnowParam_progressbar <- function()
+{
+    checkIdentical(bptasks(SnowParam()), 0L)
+    checkIdentical(bptasks(SnowParam(tasks = 0L, progressbar = TRUE)), 0L)
+    checkIdentical(bptasks(SnowParam(progressbar = TRUE)), .Machine$integer.max)
+}
+
+test_SnowParam_bpforceGC <- function() {
+    checkIdentical(FALSE, bpforceGC(SnowParam()))
+    checkIdentical(FALSE, bpforceGC(SnowParam(force.GC = FALSE)))
+    checkIdentical(TRUE, bpforceGC(SnowParam(force.GC = TRUE)))
+    checkException(SnowParam(force.GC = NA), silent = TRUE)
+    checkException(SnowParam(force.GC = 1:2), silent = TRUE)
+}
