@@ -91,3 +91,37 @@ test_bplapply_named_list <- function()
     X <- list(A = c(a = 1, b=2), B = c(c = 1, d = 2))
     checkIdentical(X, bplapply(X, identity))
 }
+
+test_bplapply_named_list_with_REDO <- function(){
+    X = setNames(1:3, letters[1:3])
+    param = SerialParam(stop.on.error = FALSE)
+    FUN1 = function(i) if (i == 2) stop() else i
+    result <- bptry(bplapply(X, FUN1, BPPARAM = param))
+    checkIdentical(names(result), names(X))
+
+    FUN2 = function(i) 0
+    redo <- bplapply(X, FUN2, BPREDO = result, BPPARAM = param)
+    checkIdentical(names(redo), names(X))
+}
+
+test_bplapply_custom_subsetting <- function(){
+    ## We have a class A in the previous unit test
+    .B <- setClass("B", slots = c(b = "integer"))
+    setMethod("[", "B", function(x, i, j, ...) initialize(x, b = x@b[i]))
+    setMethod("length", "B", function(x) length(x@b))
+    as.list.B <<- function(x, ...) lapply(seq_along(x), function(i) x[i])
+
+    x <- .B(b = 1:3)
+    expected <- lapply(x, function(elt) elt@b)
+    current <- quiet(bplapply(x, function(elt) elt@b, BPPARAM=SerialParam()))
+    checkIdentical(expected, current)
+
+    ## Remote worker does not have the definition of the class B
+    res <- tryCatch(
+        bplapply(x, function(elt) elt@b, BPPARAM=SnowParam(workers = 2)),
+        error = identity
+    )
+    checkTrue(is(res, "bplist_error"))
+
+    rm(as.list.B, inherits = TRUE)
+}
