@@ -1,11 +1,9 @@
-library(doParallel)
-
 test_bpvec_Params <- function()
 {
-    registerDoParallel(2)
+    doParallel::registerDoParallel(2)
     params <- list(serial=SerialParam(),
                    snow=SnowParam(2),
-                   batchjobs=BatchJobsParam(workers=2),
+                   batchjobs=BatchJobsParam(2, progressbar=FALSE),
                    dopar=DoparParam())
     if (.Platform$OS.type != "windows")
         params$mc <- MulticoreParam(2)
@@ -25,7 +23,7 @@ test_bpvec_Params <- function()
 }
 
 test_bpvec_MulticoreParam_short_jobs <- function() {
-    ## bpvec should return min(length(X), bpworkers())
+    ## bpvec should return min(length(X), bpnworkers())
     if (.Platform$OS.type == "windows")
         return(TRUE)
 
@@ -38,3 +36,19 @@ test_bpvec_MulticoreParam_short_jobs <- function() {
     closeAllConnections()
     TRUE
 }
+
+test_bpvec_invalid_FUN <- function() {
+    res <- bptry(bpvec(1:2, class, BPPARAM=SerialParam()))
+    checkTrue(inherits(res, "bpvec_error"))
+}
+
+test_bpvec_named_list <- function() {
+    X <- list()
+    Y <- character()
+    checkIdentical(X, bpvec(X, length))
+    checkIdentical(X, bpvec(Y, length))
+
+    names(X) <- names(Y) <- character()
+    checkIdentical(X, bpvec(X, length))
+    checkIdentical(X, bpvec(Y, length))
+}
